'use strict';
/**
 * 时间选择组件
 * @prop {boolean} show  时间选择组件是否显示
 * @prop {boolean} hourshow  小时选择条是否显示
 * @prop {boolean} minuteshow  分钟选择条是否显示
 * @prop {string} title  时间组件的标题
 * @prop {string} statusname  状态名 用于显示多少时间后开启/关闭等
 * @prop {number} hourstep  小时的间隔
 * @prop {number} minutestep 分钟的间隔
 * @prop {function} cancelClock 点击取消触发回调函数
 * @prop {function} submitClock 点击确认触发回调函数
 */

export const TimeSelect = React.createClass({
	getInitialState: function(){
        return {
        	hourtime:0,
        	minutetime:0
        };
	},
	startrange:function(e){
		//开始滑动时间刻度 记录初始坐标值
		e.stopPropagation();
		e.preventDefault();
		let yvalue = parseInt(e.touches[0].clientY);
		this.setState({
			oldy: yvalue
		});
	},
	moverange:function(e){
		//滑动时间刻度 判断滑动类型并改变刻度条的top值 产生滑动视觉效果
		e.stopPropagation();
		e.preventDefault();
		let yvalue = parseInt(e.touches[0].clientY);
		let oldy = parseInt(this.state.oldy);
		let value = (yvalue-oldy)/1.72;
		if(value>20) value=20;
		if(value<-20) value=-20;
		let type = e.target.getAttribute('data-type');
		if(type=='hour'){
			this.setState({
				newy: yvalue,
				hourtop:value
			});
		}
		if(type=='minute'){
			this.setState({
				newy: yvalue,
				minutetop:value
			});
		}
	},
	endrange:function(e){
		//滑动结束 计算滑动范围 忽略太小的滑动(20内) 然后调整选中值并重置时间刻度条
		e.stopPropagation();
		e.preventDefault();
		let newy = parseInt(this.state.newy);
		let oldy = parseInt(this.state.oldy);
		let hour = parseInt(this.state.hourtime);
		let minute = parseInt(this.state.minutetime);
		let hourstep = parseInt(this.props.hourstep) || 1;
		let minutestep = parseInt(this.props.minutestep) || 1;
		let type = e.target.getAttribute('data-type');
		if(newy-oldy>20 && type=='hour'){
			hour = hour-hourstep;
			hour = hour<0?0:hour;
			this.setState({
				hourtime:hour,
				hourtop:0
			});
		};
		if(newy-oldy<-20 && type=='hour'){
			hour = hour+hourstep;
			hour = hour>23?hour-hourstep:hour;
			this.setState({
				hourtime:hour,
				hourtop:0
			});
		};
		if(newy-oldy>20 && type=='minute'){
			minute = minute-minutestep;
			minute = minute<0?0:minute;
			this.setState({
				minutetime:minute,
				minutetop:0
			});
		};
		if(newy-oldy<-20 && type=='minute'){
			minute = minute+minutestep;
			minute = minute>59?minute-minutestep:minute;
			this.setState({
				minutetime:minute,
				minutetop:0
			});
		};
		this.setState({
			hourtop:0,
			minutetop:0
		});
	},
	endDefault:function(e){
		//阻止IOS上冒泡触发iscroll事件
		e.stopPropagation();
		e.preventDefault();
	},
	cancelclock:function(e){
		//取消选择
		if(typeof this.props.cancelClock === 'function'){
			this.props.cancelClock();
		}else{
			console.log('error:the cancel callback is not a function');
		}
	},
	submitclock:function(e){
		//确认提交时间
		if(typeof this.props.submitClock === 'function'){
			this.props.submitClock(this.state.hourtime,this.state.minutetime);
		}else{
			console.log('error:the submit callback is not a function');
		}
	},
	render: function() {
		let show = this.props.show || false;
		let hourshow = typeof this.props.hourshow !== 'undefined' && Boolean(this.props.hourshow)===false ? false : true;
		let minuteshow = typeof this.props.minuteshow !== 'undefined' && Boolean(this.props.minuteshow)===false ? false : true;
		if(!hourshow && !minuteshow) hourshow = true;
		let hourstep = parseInt(this.props.hourstep) || 1;
		let minutestep = parseInt(this.props.minutestep) || 1;
		let selecttitle = this.props.title || '设置时间';
		let statusname = this.props.statusname || '关闭';
		let hour = this.state.hourtime || '0';
		hour=parseInt(hour)>23?23:parseInt(hour);
		hour=hour<0?0:hour;
		let minute = this.state.minutetime || '0';
		minute=parseInt(minute)>59?59:parseInt(minute);
		minute=minute<0?0:minute;
		let hourtop = this.state.hourtop || 0;
		let minutetop = this.state.minutetop || 0;
	    return (
	        <section style={{display:show?'block':'none'}} className="timeselect" onTouchMove={this.endDefault}>
	        	<section className='selecttitle'>
	        		<span className='title'>{selecttitle}</span>
	        	</section>
	        	<section className='time'>
	        		<section data-type='hour' style={{left:minuteshow?'15%':'35%',display:hourshow?'inline-block':'none'}}
	        			onTouchStart={this.startrange} onTouchMove={this.moverange}
	        			onTouchEnd={this.endrange}  className='hour'>
	        		</section>
	        		<section  data-type='minute' style={{display:minuteshow?'inline-block':'none',left:hourshow?'45%':'35%'}}
	        			onTouchStart={this.startrange} onTouchMove={this.moverange}
	        			onTouchEnd={this.endrange} className='minute'>
	        		</section>
	        		<section className='timetext'>
	        			<span className='hour' style={{marginLeft:minuteshow?9.5+'rem':17+'rem',display:hourshow?'inline-block':'none'}}>时</span>
	        			<span className='minute' style={{display:minuteshow?'inline-block':'none',marginLeft:hourshow?7.33+'rem':17+'rem'}}>分</span>
	        			<span className='status'>后{statusname}</span>
	        		</section>
	        		<section className='hourvalue flex-column' style={{top:hourtop+'%',left:minuteshow?7.33+'rem':13.5+'rem',display:hourshow?'':'none'}}>
	        			<span className={hour-3*hourstep<0?'line4':'line1'}>{(hour-3*hourstep)<10?'0'+(hour-3*hourstep):hour-3*hourstep}</span>
	        			<span className={hour-2*hourstep<0?'line4':'line1'}>{(hour-2*hourstep)<10?'0'+(hour-2*hourstep):hour-2*hourstep}</span>
	        			<span className={hour-1*hourstep<0?'line4':'line2'}>{(hour-1*hourstep)<10?'0'+(hour-1*hourstep):hour-1*hourstep}</span>
	        			<span className='line3'>{hour<10?'0'+hour:hour}</span>
	        			<span className={hour+1*hourstep>23?'line4':'line2'}>{(hour+1*hourstep)<10?'0'+(hour+1*hourstep):hour+1*hourstep}</span>
	        			<span className={hour+2*hourstep>23?'line4':'line1'}>{(hour+2*hourstep)<10?'0'+(hour+2*hourstep):hour+2*hourstep}</span>
	        			<span className={hour+3*hourstep>23?'line4':'line1'}>{(hour+3*hourstep)<10?'0'+(hour+3*hourstep):hour+3*hourstep}</span>
	        		</section>
	        		<section  className='minutevalue flex-column' style={{top:minutetop+'%',display:minuteshow?'':'none',left:hourshow?15.38+'rem':13.5+'rem'}}>
		        		<span className={minute-3*minutestep<0?'line4':'line1'}>{(minute-3*minutestep)<10?'0'+(minute-3*minutestep):minute-3*minutestep}</span>
	        			<span className={minute-2*minutestep<0?'line4':'line1'}>{(minute-2*minutestep)<10?'0'+(minute-2*minutestep):minute-2*minutestep}</span>
	        			<span className={minute-1*minutestep<0?'line4':'line2'}>{(minute-1*minutestep)<10?'0'+(minute-1*minutestep):minute-1*minutestep}</span>
	        			<span className='line3'>{minute<10?'0'+minute:minute}</span>
	        			<span className={minute+1*minutestep>59?'line4':'line2'}>{(minute+1*minutestep)<10?'0'+(minute+1*minutestep):minute+1*minutestep}</span>
	        			<span className={minute+2*minutestep>59?'line4':'line1'}>{(minute+2*minutestep)<10?'0'+(minute+2*minutestep):minute+2*minutestep}</span>
	        			<span className={minute+3*minutestep>59?'line4':'line1'}>{(minute+3*minutestep)<10?'0'+(minute+3*minutestep):minute+3*minutestep}</span>
	        		</section>
	        	</section>
	        	<section className='selectbtn flex'>
	        		<span className='flex-cell' onTouchEnd={this.cancelclock}>取消</span>
	        		<span className='flex-cell' onTouchEnd={this.submitclock}>确定</span>
	        	</section>
	        </section>
	    );
	}
});